/* ---------------------------------------------------------------------- *
 * readpng.c
 * This file is part of lincity (see COPYRIGHT for copyright information).
 * ---------------------------------------------------------------------- */
#ifndef __readpng_h__
#define __readpng_h__

//int load_png_graphics (char *txt_filename, char *png_filename);
int load_png_graphics (void);

#endif /* __readpng_h__ */

/** @file gui_interface/readpng.h */

