/* ---------------------------------------------------------------------- *
 * lcstring.h
 * This file is part of lincity.
 * Lincity is copyright (c) I J Peters 1995-1997, (c) Greg Sharp 1997-2001.
 * ---------------------------------------------------------------------- */
#ifndef __lcstring_h__
#define __lcstring_h__

#include <string.h>
#ifndef WIN32
#include <unistd.h>
#endif

#ifndef _MSC_VER
//#warning deprecated header
#else
#pragma message(__FILE__ ": deprecated header")
#endif

#endif /* __lcstring_h__ */

/** @file lincity/lcstring.h */

