#ifndef ALL_MODULES_H
#define ALL_MODULES_H

#include "blacksmith.h"
#include "coalmine.h"
#include "coal_power.h"
#include "commune.h"
#include "cricket.h"
#include "fire.h"
#include "firestation.h"
#include "health_centre.h"
#include "heavy_industry.h"
#include "light_industry.h"
#include "mill.h"
#include "monument.h"
#include "oremine.h"
#include "organic_farm.h"
#include "parkland.h"
#include "port.h"
#include "pottery.h"
#include "power_line.h"
//#include "rail.h" //superseeded by track_road_rail_powerline.h
#include "recycle.h"
#include "residence.h"
//#include "road.h" //superseeded by track_road_rail_powerline.h
#include "rocket_pad.h"
#include "school.h"
#include "shanty.h"
#include "solar_power.h"
#include "substation.h"
#include "tip.h"
#include "track_road_rail.h"
//#include "track.h" //superseeded by track_road_rail_powerline.h
#include "university.h"
#include "water.h"
#include "windmill.h"
#include "windpower.h"
#include "market.h"
#include "waterwell.h"

void initializeModules();

#endif


/** @file lincity/modules/all_modules.h */

