/*
Copyright (C) 2005 David Kamphausen <david.kamphausen@web.de>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <config.h>

#include "gui_interface/shared_globals.h"

//TODO delete this file

//int module_help_flag[NUMOF_MODULES];
//int selected_module;
//int sbut[NUMOF_MODULES];
/*
void highlight_module_button (int module)
{
    (void) module;
}

void unhighlight_module_button (int button)
{
    (void) button;
}

void set_selected_module (int type)
{
    (void) type;
}
*/

/** @file lincity-ng/ButtonInterface.cpp */

