#ifndef __MODULES_INTERFACES_H__
#define __MODULES_INTERFACES_H__

void update_shanty(void);
void mps_water(int x, int y);
void add_a_shanty(void);
void remove_a_shanty(int x, int y);

//FIXME In principle all these should be obsolete
/*
void do_blacksmith(int x, int y);
void mps_blacksmith(int x, int y);

void do_coalmine(int x, int y);
void mps_coalmine(int x, int y);

void do_power_source_coal(int x, int y);
void mps_coal_power(int, int);

void do_commune(int x, int y);
void mps_commune(int x, int y);

void do_cricket(int x, int y);
void do_cricket_cover(int x, int y);
void mps_cricket(int x, int y);

void do_fire(int x, int y);

void do_firestation(int x, int y);
void do_fire_cover(int x, int y);
void mps_firestation(int x, int y);

void do_health_centre(int x, int y);
void do_health_cover(int x, int y);
void mps_health_centre(int x, int y);

void do_industry_h(int x, int y);
void mps_heavy_industry(int x, int y);

void do_industry_l(int x, int y);
void mps_light_industry(int x, int y);

void mps_market(int x, int y);
int get_food(int, int, int);
int put_food(int, int, int);
int get_jobs(int, int, int);
int put_jobs(int, int, int);
int get_goods(int, int, int);
int put_goods(int, int, int);
int get_ore(int, int, int);
int put_ore(int, int, int);
int get_coal(int, int, int);
int put_coal(int, int, int);
int add_a_market(int, int);
void remove_a_market(int, int);
void do_market(int, int);
void shuffle_markets(void);
int deal_with_transport(int, int, int, int);
int get_steel(int, int, int);
int put_steel(int, int, int);
int get_waste(int, int, int);
int put_waste(int, int, int);
int get_stuff(int, int, int, int);
int get_stuff2(map_point_info_struct *, int, int);
int get_stuff3(map_point_info_struct *, int, int);
int get_stuff4(map_point_info_struct *, int, int);
int put_stuff(int, int, int, int);
int put_stuff2(map_point_info_struct *, short int *, int, int);
int put_stuff3(map_point_info_struct *, short int *, int, int);
int put_stuff4(map_point_info_struct *, short int *, int, int);

void do_mill(int x, int y);
void mps_mill(int x, int y);

void do_monument(int x, int y);
void mps_monument(int x, int y);

void do_oremine(int x, int y);
void mps_oremine(int x, int y);

void do_organic_farm(int x, int y);
void mps_organic_farm(int x, int y);

void do_parkland(int x, int y);
void mps_parkland(int x, int y);

void do_port(int x, int y);
void mps_port(int x, int y);

void do_pottery(int x, int y);
void mps_pottery(int x, int y);

void do_power_line(int x, int y);
void mps_power_line(int x, int y);

void do_rail(int x, int y);
void mps_rail(int x, int y);

void do_recycle(int x, int y);
void mps_recycle(int x, int y);

void do_residence(int x, int y);
void mps_residence(int x, int y);

void do_road(int x, int y);
void mps_road(int x, int y);

void do_rocket_pad(int x, int y);
void launch_rocket(int x, int y);
void mps_rocket(int x, int y);

void do_school(int x, int y);
void mps_school(int x, int y);

void add_a_shanty(void);
void remove_a_shanty(int x, int y);
void update_shanty(void);
void do_shanty(int x, int y);

void do_power_source_solar(int x, int y);
void mps_solar_power(int x, int y);

void do_power_substation(int x, int y);
int add_a_substation(int x, int y);
void remove_a_substation(int x, int y);
void shuffle_substations(void);
void mps_substation(int x, int y);

void do_tip(int x, int y);
void mps_tip(int x, int y);

void do_track(int x, int y);
void mps_track(int x, int y);

void do_university(int x, int y);
void mps_university(int x, int y);

void mps_water(int x, int y);

void do_waterwell_cover(int x, int y);
void mps_waterwell(int x, int y);

void do_windmill(int x, int y);
void mps_windmill(int x, int y);
*/
#endif

/** @file lincity/modules/modules_interfaces.h */

